//-----------------------------------------------------------------------------
// TP1
// dsPIC 30F3010
// L. BAGHLI 27/03/2006
// actionner des bouton input  port
// allumer des diodes output port, PWM ADC
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
//-----------------------------------------------------------------------------
#include "p30F3010.h"

//Configuration bits
_FOSC(CSW_FSCM_OFF & XT_PLL8);
_FWDT(WDT_OFF);
_FBORPOR(PBOR_OFF & BORV_27 & PWRT_16 & MCLR_EN);
_FGS(CODE_PROT_OFF);
//-----------------------------------------------------------------------------
//Program Specific Constants
#define FCY 8000000						//Instruction cycle rate (Osc x PLL / 4)
#define FPWM 10000						//PWM freq
#define MILLISEC FCY/8000			// 1 mSec delay constant

#define StartPrg		PORTBbits.RB1
#define TogglesAlim	PORTDbits.RD0
#define EnableL6234	PORTDbits.RD1
#define RunningLED	PORTBbits.RB0
#define InterruptLED	PORTBbits.RB4
#define BugLED			PORTBbits.RB5

void setup_ports(void);
void DelayNmSec(unsigned int N);

struct {
			unsigned RunMotor : 	1;
			unsigned unused 	:		15;
		}	Flags;
unsigned int CMPR, iTime, TimerPrd, Sequence, SeqMaxDelay, SeqDelayCounter, fs;

//-----------------------------------------------------------------------------
//	Setup ports
//-----------------------------------------------------------------------------
void setup_ports(void)
{
	// Clear All Ports Prior to defining I/O
	ADPCFG = 0xFFF3;				// all PORTB = Digital(1) ;RB2 et RB3 = analog(0)  ie 0011
	PORTB=0;	//Initialize LED pin data to off state
	PORTC=0;
	PORTD=0;	//disable L6234 RD1=Off
	PORTE=0;
	// Now set pin direction registers
	TRISB = 0xFFCE;	// RB0 LED1 out, RB1 BP2 in, RB2 adc in, RB3 adc in, RB4 LED2 out, RB5 LED3 out
	TRISC = 0xDFFF;			// C13  TX out, C14 RX in, the rest is input / Unused
	TRISD =	0xFFFD; //Set pin as : RD0 BP3 input=1, RD1 output=0 enable L6234, NO debug allowed on EMUC2/EMUD2
	TRISE = 0x0000;		// RE5-0 are the PWMs -> outputs, no FLTA use
}
//-----------------------------------------------------------------------------
void InitADC10(void)
{
	// config ADC
	ADCON1 = 0x006F;				// PWM starts conversion, 4 ch simultans, ADC Off for configuring
	ADCON2 = 0x0200;				// simulataneous sample 4 channels, ADC INTerrupt  chaque EOC=100 us
	ADCHS	= 0x0022;				// Connect RB2/AN2 as CH0 : potar1 CMPR et AN3/RB3 to CH1: potar2 fs 
	ADCON3 = 0x0080;				// Tad = internal RC (4uS)
	IFS0bits.ADIF = 0;			// Adc int flag Off
	IEC0bits.ADIE = 1;			// Adc int On
	ADCON1bits.ADON = 1;			// turn ADC ON
}
//-----------------------------------------------------------------------------
//  InitMCPWM, intializes the Motor PWM as follows:
//  FPWM = 10 khz at 8Mips
//  Independant PWMs
//  Control outputs using OVDCON
//  Init Duty Cycle with a value = 100
//  Set ADC to be triggered by PWM special trigger
//-----------------------------------------------------------------------------
void InitMCPWM(void)
{
	PTPER = FCY/FPWM - 1;		// set the pwm period register
	PWMCON1 = 0x0700;			// disable PWMs
	OVDCON = 0x0000;			// allow control using OVD register
	PDC1 = 800;					// init PWM DC 1 : 800 (50% duty cycle)
	PDC2 = 0;
	PDC3 = 0;
	SEVTCMP = PTPER;		// set ADC to trigeer at ...
	PWMCON2 = 0x0000;		// 1 PWM values
	PTCON = 0x8000;			// start PWM however output ...
							// is enabled by OVDCON which is OFF
}
//---------------------------------------------------------------------
//	The ADC interrupt reads the demand pot value. 
// tt est synchrone %  cette int de 100 us
//---------------------------------------------------------------------
void __attribute__((__interrupt__)) _ADCInterrupt (void)
{
	InterruptLED=1;			// allume la diode d'interruption LED2
	IFS0bits.ADIF = 0;	// clear Int Flag
	// actionneur  plot (MRV linaire 6/4)
	CMPR	=ADCBUF0;		// lecture de la valeur du rapport cyclique
	fs		=ADCBUF1;		// lecture de la frquence de commutation des phases
	fs=fs*(fs>>4);
	PDC1=CMPR;				// mme rapport cyclique pour tout le monde
	PDC2=CMPR;
	PDC3=CMPR;
	SeqMaxDelay=fs+1;		// la vitesse de deroulement des sequences, plus fs est grande plus c'est lent !
	if (!--SeqDelayCounter)
		{
		SeqDelayCounter=SeqMaxDelay;
		if (++Sequence>=3)	Sequence=0;
		if (Sequence==0)	OVDCON=0x0300;	// phase A active only 
		if (Sequence==1)	OVDCON=0x0C00;	// phase B active only 
		if (Sequence==2)	OVDCON=0x3000;	// phase C active only 
		}	
	InterruptLED=0;	// teint la diode d'interruption LED2
}

//-----------------------------------------------------------------------------
//Main routine
int main(void)
{
unsigned int j;
	setup_ports();
	InitADC10();
	InitMCPWM();
	RunningLED=1;						// LED run = On
	RunningLED=0;						// LED run = Off
	CMPR=512;
	TimerPrd=1024;
	Sequence=0;				// init des variables du deroulement de la seq d'alim des enroulements
	SeqMaxDelay=10;
	SeqDelayCounter=SeqMaxDelay;
	if (Sequence==0)	OVDCON=0x0300;	// phase A active only 
	if (Sequence==1)	OVDCON=0x0C00;	// phase B active only 
	if (Sequence==2)	OVDCON=0x3000;	// phase C active only 
//	OVDCON = 0xFFFF;	// Les 3 PWM sont actives, no effect of OVDCON
	PWMCON1= 0x0777;					// enable PWM outputs
	EnableL6234=1;						// enable L6234
	while(1)
		{	
		}	// end of while (1)
}	// end of main

//=============================================================================
//Error traps
//-----------------------------------------------------------------------------
//Oscillator Fail Error trap routine
void _ISR _OscillatorFail(void)
{
//	LATDbits.LATD1 = 1; //Turn LED pb on
	BugLED=1;
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Address Error trap routine
void _ISR _AddressError(void)
{
	BugLED=1;
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Stack Error trap routine
void _ISR _StackError(void)
{
	BugLED=1;
	while(1); //Wait forever
}
//-----------------------------------------------------------------------------
//Math (Arithmetic) Error trap routine
void _ISR _MathError(void)
{
	BugLED=1;
	while(1); //Wait forever
}
//---------------------------------------------------------------------
// This is a generic 1ms delay routine to give a 1mS to 65.5 Seconds delay
// For N = 1 the delay is 1 mS, for N = 65535 the delay is 65,535 mS. 
// Note that FCY is used in the computation.  Please make the necessary
// Changes(PLLx4 or PLLx8 etc) to compute the right FCY as in the define
// statement above.
//---------------------------------------------------------------------
void DelayNmSec(unsigned int N)
{
unsigned int j;
while(N--)
 	for(j=0;j < MILLISEC;j++);
}
//---------------------------------------------------------------------
